ForceRequiredAddOn("Weapon_Spear");

datablock AudioProfile(DisciplineHitSound)
{
	filename    = "./hit.wav";
	description = AudioClose3d;
	preload = true;
};

datablock ItemData(DisciplineItem)
{
	category = "Weapon";
	className = "Weapon";

	shapeFile = "./hat of discipline2.dts";
	mass = 1;
	density = 0.2;
	elasticity = 0.2;
	friction = 0.6;
	emap = true;

	uiName = "Hat of Discipline";
	iconName = "./discipline2";
	doColorShift = true;
	colorShiftColor = "0.5 0.34 0.24 1.0";
	
	image = DisciplineImage;
	canDrop = true;
};

function DisciplineItem::onUse(%this, %obj, %invPosition)
{
	%obj.mountImage(DisciplineImage, 2);
	%obj.unMountImage(0);
	%obj.playThread(1, root);
}

datablock ShapeBaseImageData(DisciplineImage)
{
	shapeFile = "./hat of discipline2.dts";
	emap = true;

	mountPoint = $HeadSlot;
	offset = "0 0 0";
	rotation = eulerToMatrix("0 0 0");
	eyeOffset = "0 -0.28 0";

	correctMuzzleVector = false;

	className = "WeaponImage";

	item = DisciplineItem;
	ammo = " ";
	projectile = hammerProjectile;
	projectileType = Projectile;

	melee = true;
	doRetraction = false;
	armReady = true;

	doColorShift = true;
	colorShiftColor = DisciplineItem.colorShiftColor;

	stateName[0]					= "Activate";
	stateTimeoutValue[0]			= 0.1;
	stateTransitionOnTimeout[0]		= "Ready";
	stateSound[0]					= WeaponSwitchSound;

	stateName[1]					= "Ready";
	stateTransitionOnTriggerDown[1]	= "Fire";
	stateScript[1]                  = "onReady";
	stateAllowImageChange[1]		= true;

	stateName[2]					= "Fire";
	stateTransitionOnTimeout[2]		= "Wait";
	stateTimeoutValue[2]			= 0.2;
	stateFire[2]					= true;
	stateScript[2]					= "onFire";
	stateWaitForTimeout[2]			= true;
	stateAllowImageChange[2]		= false;

	stateName[3]					= "Wait";
	stateTransitionOnTimeout[3]		= "Ready";
	stateTimeoutValue[3]			= 0.3;
	stateWaitForTimeout[3]			= true;
	stateAllowImageChange[3]		= false;
	stateScript[3]					= "onWait";
};

function DisciplineImage::onMount(%this, %obj, %slot)
{
	Parent::onMount(%this, %obj, %slot);
	%obj.disciplineLoop();
}

function DisciplineImage::onUnMount(%this, %obj, %slot)
{
	Parent::onUnMount(%this, %obj, %slot);
	cancel(%obj.discipline);
}

function DisciplineImage::onFire(%this, %obj, %slot)
{
	
}

function Player::disciplineLoop(%obj)
{
	cancel(%obj.discipline);
	
	// getting the change in height for the hammer to determine velocity and shit
	%zspeed = mAbs(%obj.disciplinelastzpos - getWord(%obj.getPosition(), 2));
	%speed = mAbs(%obj.disciplinelastz - getWord(%obj.getMuzzlePoint(2), 2));
	%obj.disciplinelastz = getWord(%obj.getMuzzlePoint(2), 2);
	%obj.disciplinelastzpos = getWord(%obj.getPosition(), 2);
	
	%angle = getWord(VectorToEuler(%obj.getEyeVector()), 0);
	
	if(%angle < -70 && %speed * 10 > (1.5 + %zspeed) * 10 && %obj.disciplinelastangle - %angle > -5)
	{
		if(getSimTime() - %obj.lastdiscipline > 200)
		{
			%p = new Projectile()
			{
				datablock = DisciplineProjectile;
				initialPosition = %obj.getMuzzlePoint(2);
				initialVelocity = "0 0 200";
				sourceObject = %obj;
				sourceSlot = 2;
				client = %obj.client;
				
				scale = "2 2 2";
			};
			%p.explode();
			
			ServerPlay3D(DisciplineHitSound, %obj.getMuzzlePoint(2));
			
			%obj.lastdiscipline = getSimTime();
		}
	}
	
	%obj.disciplinelastangle = getWord(VectorToEuler(%obj.getEyeVector()), 0);
	
	%obj.discipline = %obj.schedule(16, disciplineLoop);
}

DeactivatePackage(Discipline);
package Discipline
{
	function serverCmdUnUseTool(%client)
	{
		if(isObject(%item = %client.player.tool[%client.player.currTool]) && %item.getName() $= "DisciplineItem")
			%client.player.unMountImage(2);
		
		Parent::serverCmdUnUseTool(%client);
	}
	
	function serverCmdUseTool(%client, %slot)
	{
		if(isObject(%item = %client.player.tool[%slot]) && %item.getName() !$= "DisciplineItem")
		{
			if(isObject(%image = %client.player.getMountedImage(2)) && %image.getName() $= "DisciplineImage")
				%client.player.unMountImage(2);
		}
		
		Parent::serverCmdUseTool(%client, %slot);
	}
};
ActivatePackage(Discipline);

AddDamageType("Discipline", '<bitmap:Add-Ons/Weapon_HatOfDiscipline/ci_discipline> %1', '%2 <bitmap:Add-Ons/Weapon_HatOfDiscipline/ci_discipline> %1', 0.2, 1);
datablock ExplosionData(DisciplineExplosion)
{
	lifeTimeMS = 150;
	
	soundProfile = spearExplosionSound;

	emitter[0] = spearExplosionEmitter;
	emitter[1] = spearExplosionEmitter2;

	faceViewer     = true;
	explosionScale = "1 1 1";

	shakeCamera = true;
	camShakeFreq = "7.0 8.0 7.0";
	camShakeAmp = "1.0 1.0 1.0";
	camShakeDuration = 0.5;
	camShakeRadius = 15.0;

	lightStartRadius = 4;
	lightEndRadius = 3;
	lightStartColor = "0.45 0.3 0.1";
	lightEndColor = "0 0 0";

	impulseRadius = 5;
	impulseForce = 2000;

	radiusDamage	= 50;
	damageRadius	= 3;
};

datablock ProjectileData(DisciplineProjectile : GunProjectile)
{
	directDamageType = $DamageType::Discipline;
	radiusDamageType = $DamageType::Discipline;
	
	explosion = DisciplineExplosion;
	uiName = "";
};

function DisciplineProjectile::radiusDamage(%this, %obj, %col, %distanceFactor, %pos, %damageAmt)
{
	if(%obj.sourceObject == %col)
		return;
	
	Parent::radiusDamage(%this, %obj, %col, %distanceFactor, %pos, %damageAmt);
}

function DisciplineProjectile::radiusImpulse(%this, %obj, %col, %distanceFactor, %pos, %impulseAmt, %verticalAmt)
{
	if(%obj.sourceObject == %col)
		return;
	
	Parent::radiusImpulse(%this, %obj, %col, %distanceFactor, %pos, %impulseAmt, %verticalAmt);
}









function eulerToVector(%ang)
{
	if(getWordCount(%ang) == 2) //Support for giving pitch and yaw, but not roll.
		%ang = getWord(%ang, 0) SPC 0 SPC getWord(%ang, 1);

	%yaw = mDegToRad(getWord(%ang, 2));
	%pitch = mDegToRad(getWord(%ang, 0));
	%x = mSin(%yaw) * mCos(%pitch) * 1;
	%y = mCos(%yaw) * mCos(%pitch);
	%z = mSin(%pitch);

	return %x SPC %y SPC %z;
}

function axisToVector(%ang)
{
	return eulerToVector(axisToEuler(%ang));
}

function vectorToEuler(%vec)
{
	%vec = vectorNormalize(%vec);
	%yaw   = mRadToDeg(mATan(getWord(%vec, 0), getWord(%vec, 1)));
	%pitch = mRadToDeg(mASin(getWord(%vec, 2)));
	return %pitch SPC 0 SPC %yaw;
}

function vectorToAxis(%vec)
{
	return eulerToAxis(vectorToEuler(%vec));
}

function vectorFloatLength(%vec, %float)
{
	return mFloatLength(getWord(%vec, 0), %float) SPC mFloatLength(getWord(%vec, 1), %float) SPC mFloatLength(getWord(%vec, 2), %float);
}

function vectorRotateVector(%vec1, %vec2)
{
	return vectorRotateAxis(%vec1, vectorToAxis(%vec2));
}

function vectorRotateEuler(%vec, %euler)
{
	return vectorRotateAxis(%vec, eulerToAxis(%euler));
}

function vectorRotateAxis(%vec, %axis)
{ //Epic function found online. Credits to Blocki <3. Id also like to thank Zeblote for finding this for me <3
	%u["x"] = getword(%axis,0);
	%u["y"] = getword(%axis,1);
	%u["z"] = getword(%axis,2);

	%angl = getword(%axis,3) * -1;
	%cos = mcos(%angl);
	%sin = msin(%angl);

	%a[1,1] = %cos + (%u["x"] * %u["x"] * (1 - %cos));
	%a[1,2] = (%u["x"] * %u["y"] * (1 - %cos)) - (%u["z"] * %sin);
	%a[1,3] = (%u["x"] * %u["z"] * (1 - %cos)) + (%u["y"] * %sin);

	%a[2,1] = (%u["y"] * %u["x"] * (1 - %cos)) + (%u["z"] * %sin);
	%a[2,2] = %cos + (%u["y"] * %u["y"] * (1 - %cos));
	%a[2,3] = (%u["y"] * %u["z"] * (1 - %cos)) - (%u["x"] * %sin);

	%a[3,1] = (%u["z"] * %u["x"] * (1 - %cos)) - (%u["y"] * %sin);
	%a[3,2] = (%u["z"] * %u["y"] * (1 - %cos)) + (%u["x"] * %sin);
	%a[3,3] = %cos + (%u["z"] * %u["z"] * (1 - %cos));

	%x = getWord(%vec, 0);
	%y = getWord(%vec, 1);
	%z = getWord(%vec, 2);

	%newx = (%a[1,1] * %x) + (%a[1,2] * %y) + (%a[1,3] * %z);
	%newy = (%a[2,1] * %x) + (%a[2,2] * %y) + (%a[2,3] * %z);
	%newz = (%a[3,1] * %x) + (%a[3,2] * %y) + (%a[3,3] * %z);

	%pos = %newx SPC %newy SPC %newz;
	return %pos;
}

function vectorLerp(%init, %end, %t)
{
	return vectorAdd(%init, vectorScale(vectorSub(%end, %init), %t));
}

function vectorSlerp(%init, %end, %t) 
{ //Thanks wikipedia! Returns spherical interpolation of two vectors
	%ang = vectorAngleBetween(%init, %end);
	//vecA * (sin((1-t)*a)/sin(a)) + vecB * (sin(t*a)/sin(a))
	return vectorAdd(vectorScale(%init, mSin((1-%t) * %ang) / mSin(%ang)), vectorScale(%end, mSin(%t * %ang) / mSin(%ang)));
}

function vectorAngleBetween(%vec1, %vec2) 
{ //Returns the angle between two vectors in radians
	return mACos(vectorDot(%vec1, %vec2)/(vectorLen(%vec1) * vectorLen(%vec2)));
}

function vectorComponent(%vec1, %vec2)
{ //Returns the component of vec1 in the direction of vec2. This is a float.
	return vectorDot(%vec1, vectorNormalize(%vec2));
}

function vectorProjection(%vec1, %vec2)
{ //Returns the vector projection, similar to component, but this is a vector
	return vectorScale(vectorNormalize(%vec2), vectorComponent(%vec1, %vec2));
}

function vectorRejection(%vec1, %vec2)
{ //Returns the vector rejection of vec1 from vec2, this is another vector
	return vectorSub(%vec1, vectorProjection(%vec1, %vec2));
}

function eulerToAxis(%euler)
{
	%euler = VectorScale(%euler,$pi / 180);
	%matrix = MatrixCreateFromEuler(%euler);
	return getWords(%matrix,3,6);
}

function axisToEuler(%axis)
{
	%angleOver2 = getWord(%axis,3) * 0.5;
	%angleOver2 = -%angleOver2;
	%sinThetaOver2 = mSin(%angleOver2);
	%cosThetaOver2 = mCos(%angleOver2);
	%q0 = %cosThetaOver2;
	%q1 = getWord(%axis,0) * %sinThetaOver2;
	%q2 = getWord(%axis,1) * %sinThetaOver2;
	%q3 = getWord(%axis,2) * %sinThetaOver2;
	%q0q0 = %q0 * %q0;
	%q1q2 = %q1 * %q2;
	%q0q3 = %q0 * %q3;
	%q1q3 = %q1 * %q3;
	%q0q2 = %q0 * %q2;
	%q2q2 = %q2 * %q2;
	%q2q3 = %q2 * %q3;
	%q0q1 = %q0 * %q1;
	%q3q3 = %q3 * %q3;
	%m13 = 2.0 * (%q1q3 - %q0q2);
	%m21 = 2.0 * (%q1q2 - %q0q3);
	%m22 = 2.0 * %q0q0 - 1.0 + 2.0 * %q2q2;
	%m23 = 2.0 * (%q2q3 + %q0q1);
	%m33 = 2.0 * %q0q0 - 1.0 + 2.0 * %q3q3;
	return mRadToDeg(mAsin(%m23)) SPC mRadToDeg(mAtan(-%m13, %m33)) SPC mRadToDeg(mAtan(-%m21, %m22));
}